

package org.hawk.ds;

import org.hawk.exception.HawkException;
import java.util.ArrayList;
import java.util.List;
import org.hawk.logger.HawkLogger;
import org.hawk.logger.HawkLogger;


/**
 * This represents the input
 * @version 1.0 15 Apr, 2010
 * @author msahu
 * @see Element
 */
public class Expression {

    private static final HawkLogger logger = HawkLogger.getLogger(Expression.class.getName());

    /**
     * This is the placeholder to store the input string in list of Element
     */
    private List<Element>  expression;

    /**
     * This represents the type of the input expression.
     * INFIX,POSTFIX,PREFIX.
     */
    private String  type;

    /**
     * CTOR ... The type is considered as INFIX.
     * @param elements list of elements that represents this expression.
     */
    public Expression(final List<Element> elements ){
        this(elements , HawkDSConstant.INFIX);

    }

    public List<Element> getExpression() {
        return expression;
    }

    public void setExpression(List<Element> expression) {
        this.expression = expression;
    }
    

    /**
     * CTOR considering the type as well.
     * Allowed type  being INFIX,POSTFIX AND PREFIX.
     * @param elements
     * @param type
     */
    public Expression(final List<Element> elements ,final String type){
        this.expression= elements;
        this.type=type;
    }
    public String getType(){return type;}

    public static void main(String args[]) throws Exception{

        String input = "(`exec fib(1.0)`+2)+`exec fib(1.0)`";
        List<Element> elements = Element.parseElements(input);
        Expression infix = new Expression(elements);

        //logger.info("op ="+infix);
        logger.info(infix.toPrefix().toString());


    }
    /**
     * This converts the input expression into PREFIX format.
     * @return returns the prefix format of the input expression.
     * @throws HawkException
     */
    public List<Element> toPrefix() throws HawkException{
        if(!type.equals(HawkDSConstant.INFIX)){
            throw new HawkException(" Expecting infix  found "+type);
        }
        try{

            this.expression =Element.reverse(this.expression);
            Expression tmp = new Expression(this.expression);

            tmp.toPostfix();

            this.expression = Element.reverse(tmp.expression);
            this.type=HawkDSConstant.PREFIX;
        }catch(Throwable th){

            throw new HawkException("Invalid ",th);
        }
        return new ArrayList(expression);
    }

    /**
     * This returns the the POSTFIX form of the input expression.
     * @return returns the POSTFIX form of the input expression.
     * @throws HawkException
     */
    public List<Element> toPostfix() throws HawkException{

        if(!type.equals(HawkDSConstant.INFIX)){
            throw new IllegalStateException(" Expecting infix  found "+type);
        }


        final int infix_len = this.expression.size();
        List<Element> postfix = new ArrayList<Element>();
        int i = 0;
        final Stack<Element> stack = new Stack<Element>();

        while (i < infix_len) {

            Element element = this.expression.get(i);

            if ( element.isClubber()  && element.isStartingClubber()) {

                stack.push(element);

            } else if (element.isOperator()){
                final Element top = stack.top();
                if (top != null) {
                    if (top.isOperator()){
                        if(top.hasHigherPrecedence(element)){
                            stack.pop();
                            postfix.add(top);
                            stack.push(element);
                            }else{
                                stack.push(element);
                            }
                    }else{
                            stack.push(element);
                    }
                }else{
                        stack.push(element);
                }
            } else if (element.isClubber()  &&  element.isClosingClubber()){

                Element top =  stack.top();

                while (!(top.isClubber()  && top.isStartingClubber())){
                        stack.pop();
                        postfix.add(top);
                        top =  stack.top();

                }
                stack.pop();

            } else {

                postfix.add(element);
            }

            i++;
        }

        Element top = stack.top();

        while (top != null) {
            stack.pop();
            postfix.add(top);
            top = stack.top();
        }

        if(checkForClubbers(postfix)){
            throw new HawkException("Invalid...");
        }
        this.expression = postfix;
        this.type=HawkDSConstant.POSTFIX;
        return this.expression;
    }

    /**
     * This searches for "(" and ")" in the input elements if one of them is present in the
     * input elements.
     * @param elements
     * @return
     */
    private static  boolean checkForClubbers(List<Element> elements){

        boolean status = false;
        if(elements == null || elements.isEmpty()){
            return false;
        }
        for(Element element : elements){
            if(element.isClubber()){
                status = true;
                break;
            }
        }

        return status;
    }

	
}




