

package org.hawk.module;


import org.hawk.logger.HawkLogger;
import org.hawk.module.annotation.Module;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.TreeMap;

import java.util.logging.Level;
import org.hawk.module.annotation.Implementor;
import org.hawk.module.annotation.Implementors;
import static org.hawk.constant.HawkConstant.*;
/**
 *
 * @version 1.0 30 Mar, 2010
 * @author msahu
 */
public class ModuleFactory {
   private static final HawkLogger logger = HawkLogger.getLogger(ModuleFactory.class.getName());

   private static Map<String,IModule> cachedModules = new LinkedHashMap<String,IModule>();


   public static String getModuleName(IModule module){
       if(module == null){
           return null;
       }
       Class moduleClazz = module.getClass();
       String moduleName = moduleClazz.getSimpleName();
       if(moduleClazz.isAnnotationPresent(Module.class)){

            Module moduleAnnotation = (Module) moduleClazz.getAnnotation(Module.class);
            moduleName = moduleAnnotation.name();
        }

       return moduleName;
   }

   /**
    * This caches all the Hawk modules.
    * Essentially these includes target module's IModule implementation and hawk's internal
    * library modules.
    * @param props the hawk property file.
    * @return
    */
   public static Map<String,IModule> cacheModules(Properties props){

       Map<Integer,IModule> map = new TreeMap<Integer,IModule>();
       cacheLibraryModules(map);
       cacheTargetAppModules(props, map);

       return cachedModules;

   }

   /**
    * This caches all the hawk library modules.
    * @param map
    * @return
    */
   public static boolean cacheLibraryModules(Map<Integer,IModule> map){

        Class libModuleClazz = LibraryModule.class;

        if(libModuleClazz.isAnnotationPresent(Implementors.class)){
            Implementors implementors =(Implementors) libModuleClazz.getAnnotation(Implementors.class);

            for(Implementor implementor :implementors.value()){
                try {
                        String clazzStr  = "org.hawk.module.lib."+implementor.clazz();

                        Class clazz = Class.forName(clazzStr);
                        IModule instance = (IModule) clazz.newInstance();

                        map.put(map.size(), instance);

                } catch (Exception ex) {
                    ex.printStackTrace();
                }

            }

        }

       for(Entry<Integer,IModule> entry:map.entrySet()){
            cachedModules.put(getModuleName(entry.getValue()), entry.getValue());
       }



       return true;
   }

   /**
    * This caches target module's all IModule implementations.
    * @param props
    * @param map
    * @return
    */
   private static boolean cacheTargetAppModules(Properties props,Map<Integer,IModule> map){

        for(int i=1;i<Integer.MAX_VALUE;i++){

           String moduleImplKey = TARGET_MODULE_IMPLEMENTOR_KEY+SEPARATOR+i;
           String clazzStr = props.getProperty(moduleImplKey);

           if(clazzStr == null || clazzStr.isEmpty()){
               break;
           }

           String modulePackage = props.getProperty(TARGET_MODULE_PACKAGE_KEY);
           clazzStr = modulePackage + "." + clazzStr;
           try{
               Class clazz = Class.forName(clazzStr);
               IModule instance = (IModule) clazz.newInstance();
               map.put(i+map.size()-1, instance);
           }catch(Exception ex){
               logger.log(Level.SEVERE,"Could not create new module instance ",ex);
               return false;
           }


       }

       for(Entry<Integer,IModule> entry:map.entrySet()){
            cachedModules.put(getModuleName(entry.getValue()), entry.getValue());
       }

       return true;
   }



   /**
    * This returns all the hawk modules.
    * If the cache is empty , it fills it up.
    *
    * @param props
    * @return
    */
   public static Map<String,IModule> getModules(Properties props){
       if(cachedModules == null || cachedModules.isEmpty()){
           return cacheModules(props);
       }
       return cachedModules;
   }

   /**
    * This returns the cached module.
    * This does not check for cache emptiness.
    * @return
    */
   public static Map<String,IModule> getModules(){
       
       return cachedModules;
   }
   /**
    * 
    * @param updatedModules
    */
   public static void setModules(Map<String,IModule> updatedModules){
       cachedModules = updatedModules;
   }
}




