

package org.hawk.module.script;


import org.hawk.module.script.FunctionScript.FunctionInvocationInfo;
import java.util.Map.Entry;
import org.hawk.ds.Stack;
import org.hawk.exception.HawkException;
import org.hawk.module.annotation.CreateScript;
import org.hawk.module.annotation.ScriptPattern;
import java.util.Map;
import java.util.TreeMap;
import org.hawk.logger.HawkLogger;
import java.util.regex.Pattern;
import org.hawk.module.lib.SystemModule;
import org.hawk.module.script.type.BooleanDataType;
import org.hawk.module.script.type.Variable;
import static org.hawk.constant.HawkLanguageKeyWord.*;

/**
 *
 * @version 1.0 11 Apr, 2010
 * @author msahu
 */
public class ExecFunctionScript extends SingleLineScript{

    private static final HawkLogger logger = HawkLogger.getLogger(ExecFunctionScript.class.getName());
    private  static final Pattern FUNCTION_EXEC_PATTERN=
    Pattern.compile("\\s*"+exec+"\\s*([a-z|A-Z]{1,}[a-z|A-Z|\\d]*)\\s*\\(\\s*(.*)\\s*\\)\\s*");

    /**
     * ExecFunctionScript pattern.
     * @return returns ExecFunctionScript
     * @see ScriptPattern
     */
    @ScriptPattern
    public static Pattern getPattern(){
        return FUNCTION_EXEC_PATTERN;
    }

    @CreateScript
    public static ExecFunctionScript createScript(Map<Integer,String> lineFunctionMatcherMap) throws HawkException{

        if(lineFunctionMatcherMap == null){
            return null;
        }
        ExecFunctionScript functinScript = new ExecFunctionScript();
        functinScript.setFunctionName(lineFunctionMatcherMap.get(1));
        functinScript.setParams(lineFunctionMatcherMap.get(2));
        

        return functinScript;
    }

    protected  String parseFunctionParameters() throws HawkException{

        this.paramMap.clear();
        StringBuilder sb = new StringBuilder();
        this.params = this.params.trim();
        if(this.params.endsWith(",")){
            throw new HawkException("Invalid parameters");
        }

        if(!this.params.isEmpty())
        {
            String paramsArr[] = this.params.split(",");

            if(paramsArr == null || paramsArr.length == 0 ){
                throw new HawkException("Invalid parameters");

            }else{
                int i = 1;
       
                for(String param :paramsArr){
                   IScript result = this.evaluateLocalVariable(param);
                                                            
                   this.paramMap.put(i,result);
                   sb.append(result.mangle());
                   i++;
                }
            }

        }

        return sb.toString();
    }


    private String functionName;

    private String params ;

    private Map<Integer,IScript> paramMap = new TreeMap<Integer,IScript>();

    public Map<Integer,IScript> getParamMap(){
        return this.paramMap;
    }
    public Map<Integer, IScript> cloneParamMap() {
        
        logger.info("Executing in thread {"+Thread.currentThread().getName()+"}");
        Map<Integer,IScript> clonedParamMap = new TreeMap<Integer,IScript>();
        for(Entry<Integer,IScript> entry:this.paramMap.entrySet()){
            clonedParamMap.put(entry.getKey(),entry.getValue().copy());
        }

        return clonedParamMap;
        
    }

    public void setParamMap(Map<Integer, IScript> paramMap) {
        this.paramMap = paramMap;
    }

    

   

    public String getParams() {
        return params;
    }

    public void setParams(String params) {
        this.params = params;
    }

    

    public String getFunctionName() {
        return functionName;
    }

    public void setFunctionName(String functionName) {
        this.functionName = functionName;
    }

    

    @Override
    public Variable execute() throws HawkException {

        this.paramMap.clear();
        
        String mangledParams = this.parseFunctionParameters();
        String mangledFunctionName = this.functionName+mangledParams;
        Variable rtnVar =  execute(functionName, this.getLineNumber(), mangledFunctionName, paramMap);
        if(!this.isLastScript()){
            rtnVar.setRtrn(false);
        }
        return rtnVar;

    }
    public static Variable execute
            (
                String functionName,
                int lineNumber,
                String mangledFunctionName,
                Map<Integer,IScript> paramMap
            ) throws HawkException{

        FunctionScript functionScript = ScriptInterpreter.getInstance()
                                                         .findFunctionScript(mangledFunctionName);

        FunctionInvocationInfo functionInvocationInfo =new FunctionInvocationInfo(functionName,lineNumber);

                
        Stack<FunctionScript.FunctionInvocationInfo> functionStack = SystemModule.getFunctionStack().get();
        if(functionStack == null){
            functionStack = new Stack<FunctionScript.FunctionInvocationInfo>();
        }
        functionStack.push(functionInvocationInfo);
        SystemModule.getFunctionStack().set(functionStack);

        Variable rtn = new Variable();
        rtn.setVariableValue(new BooleanDataType(false));
        rtn = functionScript!= null
                    ?
                            functionScript.executeDefaultForLoopScript(paramMap)
                        :
                            rtn
                        ;
        functionStack.pop();
        SystemModule.getFunctionStack().set(functionStack);
        return rtn;
    }

    @Override
    public boolean isVariable() {
        return false;
    }
}




