

package org.hawk.module.script;

import org.hawk.util.HawkUtil;
import org.hawk.exception.HawkException;

import org.hawk.module.IModule;
import org.hawk.module.ModuleFactory;
import org.hawk.module.annotation.CreateScript;
import org.hawk.module.annotation.ScriptPattern;
import org.hawk.module.script.enumeration.BuildModeEnum;
import org.hawk.module.task.SubTaskContainer;
import java.util.Map;
import java.util.logging.Level;
import org.hawk.logger.HawkLogger;
import java.util.regex.Pattern;
import org.hawk.module.script.type.StringDataType;
import org.hawk.module.script.type.Variable;
import static org.hawk.constant.HawkLanguageKeyWord.*;


/**
 *
 * @version 1.0 8 Apr, 2010
 * @author msahu
 */
public class ExecModuleScript extends SingleLineScript{

    private  static final Pattern EXEC_PATTERN=
     Pattern.compile("\\s*"+exec+"\\s*([a-z|A-Z]{1,}[a-z|A-Z|\\d]*)->([a-z|A-Z]{1,}[a-z|A-Z|\\d]*)\\s*\\(\\s*(.*)\\s*\\)\\s*");

    public ExecModuleScript(){

    }


    private String params ;

    

    public String getParams() {
        return params;
    }

    public void setParams(String params) {
        this.params = params;
    }


    private  boolean parseModuleParameters() throws HawkException{

            this.subTaskContainer.clearParams();
            
            this.params = this.params.trim();
            if(this.params.endsWith(",")){
                throw new HawkException("Invalid parameters");
            }

            if(!this.params.isEmpty())
            {
                String paramsArr[] = this.params.split(",");

                if(paramsArr == null || paramsArr.length == 0 ){
                    throw new HawkException("Invalid parameters");

                }else{
                    int i = 1;

                    for(String param :paramsArr){

                        IScript resultScript  =this.evaluateLocalVariable(param);
                                                                
                        Object paramObj = ""+resultScript.getVariableValue();
                        if(resultScript instanceof StructureScript){
                            StructureScript structureScript = (StructureScript)resultScript;
                            paramObj = structureScript.toMap();
                        }else if(resultScript instanceof LocalVarDeclScript){
                            Variable result = resultScript.getVariableValue();
                            if(result.getVariableValue() instanceof StringDataType){
                                String paramStr = ((StringDataType)result.getVariableValue()).getData();
                                paramStr = HawkUtil.convertHawkStringToJavaString(paramStr);
                                paramObj = paramStr;
                            }else{
                                paramObj = resultScript.getVariableValue();
                            }
                        }
                        this.subTaskContainer.addParam(paramObj);
                        i++;
                    }
                }

            }

            return true;
    }
   
    @ScriptPattern
    public static Pattern getPattern() {
        return EXEC_PATTERN;
    }

    @CreateScript
    public static ExecModuleScript createScript(Map<Integer,String> lineModuleMatcherMap) throws HawkException{

        if(lineModuleMatcherMap == null){
            return null;
        }

        ExecModuleScript execScript = new ExecModuleScript();
        String moduleName = lineModuleMatcherMap.get(1);

        String moduleSubTask = lineModuleMatcherMap.get(2);

        String moduleParams = lineModuleMatcherMap.get(3);

        Map<IModule,Map<String,SubTaskContainer>> cachedSubTasks = ScriptInterpreter.getInstance().getSubTasks();
        Map<String,IModule> moduleMap = ModuleFactory.getModules();

        IModule moduleInstance = moduleMap.get(moduleName);
        if (moduleInstance == null){
            //Look for alias
            AliasScript aliasScript= AliasScript.getInstance();
            moduleInstance = moduleMap.get(aliasScript.get(moduleName));
        }
        if(moduleInstance == null){
            throw new HawkException("Could not find the module {"+moduleName+"}");
        }
        SubTaskContainer subTaskContainer = cachedSubTasks.get(moduleInstance).get(moduleSubTask);
        if(subTaskContainer == null){
            throw new HawkException("Invalid task{"+moduleName+"->"+moduleSubTask+"}");
        }
        subTaskContainer.setTaskName(moduleSubTask);
        subTaskContainer.setTaskIteration(1);
        subTaskContainer.setModule(moduleName);
        execScript.setSubTaskContainer(subTaskContainer);
        execScript.setParams(moduleParams);
        return execScript;
    }


    private SubTaskContainer subTaskContainer;

    public SubTaskContainer getSubTaskContainer() {
        return subTaskContainer;
    }

    public void setSubTaskContainer(SubTaskContainer subTaskContainer) {
        this.subTaskContainer = subTaskContainer;
    }

    
    
    @Override
    public Variable execute() throws HawkException {

        BuildModeEnum buildMode = ScriptUsage.getInstance().getBuildMode();
        logger.log(Level.INFO,"Executing "+this.subTaskContainer.getTaskName());
        this.parseModuleParameters();
        Variable status = null;
        status = this.subTaskContainer.execute();
        return status;
    }

    @Override
    public String toString(){
        return "";
    }
    @Override
    public boolean isVariable() {
        return false;
    }

    private static final HawkLogger logger = HawkLogger.getLogger(ExecModuleScript.class.getName());
}




