

package org.hawk.module.script;

import org.hawk.exception.HawkException;
import org.hawk.module.annotation.CreateScript;
import org.hawk.module.annotation.ScriptPattern;
import java.util.Map;
import java.util.regex.Pattern;
import org.hawk.module.script.enumeration.VarTypeEnum;
import org.hawk.module.script.type.StringDataType;
import org.hawk.module.script.type.Variable;
import org.hawk.util.HawkUtil;
import static org.hawk.constant.HawkLanguageKeyWord.*;

/**
 *
 * @version 1.0 29 Apr, 2010
 * @author msahu
 */
public class LocalVarDeclScript extends VariableDeclScript {

   

    public LocalVarDeclScript(){
        super();
    }
   
    public String getLocalVarValExp() {
        return this.vaiableValExp;
    }

    public void setLocalVarValExp(String localVarValExp) {
        this.vaiableValExp = localVarValExp;
    }

    

    public void setLocalVar(Variable localVar) {
        this.variable = localVar;
    }

    public void setLocalVarValue(Variable localVarValue) {
        this.variableValue = localVarValue;
    }

    public Variable getLocalVar() {
        return variable;
    }

    public Variable getLocalVarValue() {
        return this.variableValue;
    }

    @Override
    public String toString(){
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append
                (
                    this.getLocalVar()!=null
                                        ?
                                                this.getLocalVar().getVariableName()
                                            :   ""
                );
        sb.append
                (
                    this.getLocalVarValue()!=null && this.getLocalVarValue().getVariableValue() != null
                                            ?
                                                    this.getLocalVarValue().getVariableValue()
                                                :
                                                    ""
                );
        sb.append("}");
        return sb.toString();
    }


    @Override
    public LocalVarDeclScript copy(){
        LocalVarDeclScript copy = new LocalVarDeclScript();
        copy.setLocalVar(this.getLocalVar());
        copy.setLocalVarValExp(this.getLocalVarValExp());
        copy.setLocalVarValue(this.getLocalVarValue());
        copy.setOuterMultiLineScript(this.getOuterMultiLineScript());
        return copy;
    }


    @ScriptPattern
    public static Pattern getPattern() {
        return VARIABLE_PATTERN;
    }
    @Override
    public Variable execute() throws HawkException {


        Variable result = null;
        if(!HawkUtil.isStringDataType(this.vaiableValExp)){
            
            result = this.evaluateLocalVariable(this.vaiableValExp).getVariableValue();
            this.variableValue = result;
        }else{
            result = new Variable(VarTypeEnum.VAR,null,this.variable.getVariableName());
            result.setVariableValue(new StringDataType(this.vaiableValExp));
            this.variableValue = result;
        }
        this.outerMultiLineScript.setLocalValue(this.variable, this);
        this.variable.setVariableValue(result.getVariableValue());
        return this.variable;
    }

    @CreateScript
    public static LocalVarDeclScript createScript(Map<Integer,String> lineLocalVarDeclMatcherMap){

        if(lineLocalVarDeclMatcherMap == null){
            return null;
        }
        LocalVarDeclScript localVarDeclScript = new LocalVarDeclScript();
        Variable localVar = new Variable(VarTypeEnum.VAR,null,lineLocalVarDeclMatcherMap.get(1));
        localVarDeclScript.setLocalVar(localVar);
        localVarDeclScript.setLocalVarValExp(lineLocalVarDeclMatcherMap.get(2));
        return localVarDeclScript;
    }

    
    public static boolean isValid(Variable variable,MultiLineScript multiLineScript) throws HawkException {
        
        if(variable == null ||variable.getVariableName() == null || variable.getVariableName().isEmpty() || multiLineScript == null){
            throw new HawkException("invalid parameter");
        }
        
        boolean  operandDeclared = false;
        do{
            operandDeclared = multiLineScript.isLocalVarDeclared(variable);
            if(operandDeclared){
                return false;
            }else{
            
                if (multiLineScript.getFunctionScript() != null) {
                    operandDeclared = multiLineScript.getFunctionScript()
                                                     .isParamVarDeclared(variable);
                    return !operandDeclared
                            ?
                                    !ScriptInterpreter.getInstance()
                                                      .isGlobalVarDeclared(variable)
                                :
                                    false;
                }
            
                multiLineScript = multiLineScript.getOuterMultiLineScript();
            }
        }while (true);
    }

     @Override
    public boolean isVariable() {
        return true;
    }

    @Override
    public Variable getVariable() {
        return super.getVariable();
    }

    @Override
    public void setVariableValue(Variable variableValue) {
        super.setVariableValue(variableValue);
    }

    @Override
    public String mangle(){

        return "_"+var+"_";
    }
}


