


package org.hawk.module.script;


import org.hawk.exception.HawkException;
import org.hawk.module.annotation.CreateScript;
import org.hawk.module.annotation.Implementor;
import org.hawk.module.annotation.Implementors;
import org.hawk.module.annotation.ScriptPattern;
import org.hawk.module.annotation.SingleTonSetter;
import org.hawk.pattern.PatternMatcher;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.Map.Entry;
import org.hawk.logger.HawkLogger;
import java.util.regex.Pattern;
import org.hawk.module.script.enumeration.BuildModeEnum;
import org.hawk.module.script.type.Variable;

/**
 *
 * @version 1.0 10 Apr, 2010
 * @author msahu
 */

@Implementors
(
    {

            
            @Implementor(clazz="EchoScript",parsingSequence=1),
            @Implementor(clazz="ThinkScript",parsingSequence=2),
            @Implementor(clazz="ExecFunctionScript",parsingSequence=3),
            @Implementor(clazz="ExecModuleScript",parsingSequence=4),
            @Implementor(clazz = "ArrayDeclScript",parsingSequence=5),
            @Implementor(clazz = "BreakScript",parsingSequence=6),
            @Implementor(clazz = "ReturnScript",parsingSequence=7),
            @Implementor(clazz = "StructureScript",parsingSequence=8),
            @Implementor(clazz = "ExecParallelSingleLineScript",parsingSequence=9),
            @Implementor(clazz = "ExecBackgroundSingleLineScript",parsingSequence=10),
            @Implementor(clazz = "ReadLineScript",parsingSequence=11),
            @Implementor(clazz = "LocalVarDeclScript",parsingSequence=12),
            @Implementor(clazz = "CommentScript",parsingSequence=13),
            @Implementor(clazz = "AssignmentScript",parsingSequence=14)
    }
)
public  class SingleLineScript extends AbstractScript{

    private static final HawkLogger logger = HawkLogger.getLogger(SingleLineScript.class.getName());
    public Variable execute() throws HawkException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public SingleLineScript(){
        this.outerMultiLineScript = new MultiLineScript();
    }

    public static boolean addScripts(int i , String scriptStr ,MultiLineScript  multiLineScript) throws HawkException{

        boolean scriptFound = false;
        SingleLineScript script = null;

        try{
            scriptFound = addSingleLineScripts(i, scriptStr, multiLineScript);
        }catch(HawkException ex){
             throw new HawkException ("Unrecognized characters {"+scriptStr+"} at line no {"+(++i)+"}");
        }

        if(!scriptFound && (scriptStr!=null && !scriptStr.trim().isEmpty())){
            throw new HawkException ("Unrecognized characters {"+scriptStr+"} at line no {"+(++i)+"}");
        }
        return true;
    }

    private static boolean addSingleLineScripts(int i , String scriptStr,MultiLineScript multiLineScript) throws HawkException{

        Map<Class,SingleLineScript> cachedSingleLineScripts = SingleLineScriptFactory.getSingleLineScripts();

        for(Entry<Class,SingleLineScript> entry:cachedSingleLineScripts.entrySet()){

            Method [] methods = entry.getKey().getDeclaredMethods();

            Method scriptPatternMethod = null;
            Method createScriptMethod = null;
            Method singleTonSetterMethod = null;

            for(Method method : methods){

                if(method.isAnnotationPresent(ScriptPattern.class)){
                    scriptPatternMethod = method;
                }

                if(method.isAnnotationPresent(CreateScript.class)){
                    createScriptMethod  = method;
                }
                if(method.isAnnotationPresent(SingleTonSetter.class)){
                    singleTonSetterMethod  = method;
                }

            }
            try {
                Pattern pattern  = (Pattern) scriptPatternMethod.invoke(entry.getValue(), new Object[]{});
                Map<Integer,String> lineMatcherMap = PatternMatcher.match(pattern, scriptStr);
                if(lineMatcherMap != null){
                    SingleLineScript  script = (SingleLineScript)createScriptMethod.invoke(entry.getValue(), lineMatcherMap);
                    if(script!= null && script instanceof CommentScript){
                        return true;
                    }
                    if(singleTonSetterMethod!= null && script!= null){
                        singleTonSetterMethod.invoke(entry.getValue(), script);
                    }

                    if(script != null){
                        script.setLineNumber(i+1);
                        script.setOuterMultiLineScript(multiLineScript);

                        //The other option is to catch during interpretation phase
                        if (script instanceof LocalVarDeclScript){
                            LocalVarDeclScript localVarDeclScript = (LocalVarDeclScript)script;
                            if(LocalVarDeclScript.isValid(localVarDeclScript.getLocalVar(), multiLineScript)){
                                multiLineScript.addScript(i, script);
                                multiLineScript.setLocalValue(localVarDeclScript.getLocalVar(), null);
                            }else{
                                throw new HawkException(localVarDeclScript.getLocalVar()+" is already declared");
                            }
                        }else{
                            multiLineScript.addScript(i, script);
                        }
                        return true;
                    }
                }
            } catch (Throwable ex) {
                if(ScriptUsage.getInstance().getBuildMode() != BuildModeEnum.PERF){
                    ex.printStackTrace();
                }
                logger.severe(ex.getCause().getMessage());
                throw new HawkException(ex);
            }
        }
        return false;
    }

    @Override
    public boolean isVariable() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Variable getVariable() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void setVariableValue(Variable value) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public Variable getVariableValue() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void setVariable(Variable value) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public IScript copy() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    protected MultiLineScript findNearestOuterMLScript() {
        return this.getOuterMultiLineScript();
                
    }

    @Override
    public String toUI() {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}




