

package org.hawk.module.script;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Pattern;
import org.hawk.ds.Stack;
import org.hawk.exception.HawkException;
import org.hawk.module.script.type.Variable;
import org.hawk.pattern.PatternMatcher;
import static org.hawk.constant.HawkLanguageKeyWord.*;

/**
 *
 * @version 1.0 19 Jul, 2010
 * @author msahu
 */
public class StructureDefnScript extends MultiLineScript{

    private  static final Pattern STRUCTURE_DEFN_PATTERN=
            Pattern.compile("\\s*"+struct+"\\s*([a-z|A-Z]{1,}[a-z|A-Z|\\d]*)\\s*");

    private static final Pattern STRUCTURE_DEFN_BRACKET_START_PATTERN=Pattern.compile("\\s*\\{\\s*");

    private static final Pattern STRUCTURE_DEFN_BRACKET_END_PATTERN=Pattern.compile("\\s*\\}\\s*");

    

    private String structureName;

    public String getStructureName() {
        return structureName;
    }

    public void setStructureName(String structureName) {
        this.structureName = structureName;
    }

    @Override
    public String toString(){

       StringBuilder sb = new StringBuilder(this.structureName+" members are :::");

       for(Entry<Integer,IScript> entry:this.innerScripts.entrySet()){

           sb.append(entry.getValue().toString());

       }

       return sb.toString();

    }

    private static boolean contains( List<IScript> all, Variable var){
        for(IScript script:all){
            if(script.getVariable().equals(var)){
                return true;
            }
        }
        return false;
    }
    public Map<String,IScript> instantiate() throws HawkException{


        Map<String,IScript>  structInstance = new HashMap<String,IScript>();

        for(Entry<Integer,IScript> entry:this.innerScripts.entrySet()){

            IScript script = entry.getValue();

            structInstance.put(script.getVariable().getVariableName(),script.copy());

       /*     if(script instanceof LocalVarDeclScript){

                LocalVarDeclScript lvds = (LocalVarDeclScript)script;

                if(!contains(structInstance,lvds.getLocalVar())){

                    structInstance.put(lvds.getVariable().getVariableName(),lvds.copy());
                                    
                }else{

                    throw new HawkException("Duplicate key {"+lvds.getLocalVar()+"} resolved at line no {"+entry.getKey()+"}");

                }
            }else if(script instanceof StructureScript){

                StructureScript structScript = (StructureScript)script;
                structInstance.put(structScript.getVariable().getVariableName(),structScript.copy());
            }
        * 
        */

        }

        return structInstance;
    }

    /**
     * This converts the hawk structure into java map..
     * @return
     * @throws org.hawk.exception.HawkException
     */
    @Override
    public Map<Object,Object> toMap() throws HawkException{

        Map<String,IScript> instance = this.instantiate();

        Map<Object,Object> javaMap = new LinkedHashMap<Object,Object>();

        for(Entry<String,IScript> entry:instance.entrySet()){

            javaMap.put(entry.getValue().getVariable().getVariableName(), entry.getValue().getVariableValue()+"");

        }

        return javaMap;
    }

    public static Map<String,StructureDefnScript>  parseStructures(Map<Integer,String> scriptMap) throws HawkException{

        Map<String,StructureDefnScript> structureMap = new HashMap<String,StructureDefnScript>();

        for(int i=0;i<scriptMap.size();){

            Map<Integer,String> map = PatternMatcher.match(STRUCTURE_DEFN_PATTERN, scriptMap.get(i));

            if(map!= null){

                String structureName = map.get(1);

                MultiLineContainer structureDefn = extractStructureDefinition(scriptMap,i+1,structureName);

                StructureDefnScript structureDefnScript = new StructureDefnScript();

                structureDefnScript.setStructureName(structureName);

                structureDefnScript.setMultiLineContainer(structureDefn);

                structureMap.put(structureName, structureDefnScript);

                i = structureDefn.getEnd()+1;

            }else{

                i = i+1;

            }

        }

        return structureMap;
    }

   public static void parseStructMembers( Map<Integer, String> scriptMap, StructureDefnScript structureDefnScript) throws HawkException {
        MultiLineContainer structureDefn = structureDefnScript.getMultiLineContainer();

        for (int j = structureDefn.getStart() + 1; j < structureDefn.getEnd(); j++) {

            SingleLineScript singleLineScript = null;

            Map<Integer, String> lineLocalVarMatcherMap = PatternMatcher.match(LocalVarDeclScript.getPattern(), scriptMap.get(j));

            if (lineLocalVarMatcherMap == null) {

                Map<Integer, String> lineStructMatcherMap = PatternMatcher.match(StructureScript.getPattern(), scriptMap.get(j));

                if (lineStructMatcherMap != null) {

                    singleLineScript = StructureScript.createScript(lineStructMatcherMap);

                    structureDefnScript.addScript(j, singleLineScript);

                    if (singleLineScript != null) {

                        singleLineScript.setOuterMultiLineScript(structureDefnScript);

                    }

                }else{
                    Map<Integer, String> lineArrayDeclMatcherMap = PatternMatcher.match(ArrayDeclScript.getPattern(), scriptMap.get(j));

                    if (lineArrayDeclMatcherMap != null) {

                        singleLineScript = ArrayDeclScript.createScript(lineArrayDeclMatcherMap);

                        structureDefnScript.addScript(j, singleLineScript);

                        if (singleLineScript != null) {

                            singleLineScript.setOuterMultiLineScript(structureDefnScript);

                        }

                    }
              }

            } else {

                singleLineScript = LocalVarDeclScript.createScript(lineLocalVarMatcherMap);

                singleLineScript.execute();

                structureDefnScript.addScript(j, singleLineScript);

                if (singleLineScript != null) {

                    singleLineScript.setOuterMultiLineScript(structureDefnScript);

                }
            }
        }
    }


   private static MultiLineContainer extractStructureDefinition(Map<Integer,String> scriptMap,int x,String structureName) throws HawkException{

        MultiLineContainer mlc = new MultiLineContainer();

        boolean firstStartBracktFound = false;

        Stack stack = new Stack();

        int start = x;

        int end = -1;

        int i = x;

        for(;i<scriptMap.size();i++){

            Map<Integer,String> startingBracketMap = PatternMatcher.match
                                                                    (
                                                                        STRUCTURE_DEFN_BRACKET_START_PATTERN,
                                                                        scriptMap.get(i)
                                                                    );

            if(startingBracketMap!=null){

                    if(!firstStartBracktFound){

                        firstStartBracktFound = true;

                        start = i;
                    }

                    stack.push("{");

            }

            Map<Integer,String> endingBracketMap = PatternMatcher.match
                                                                    (
                                                                        STRUCTURE_DEFN_BRACKET_END_PATTERN,
                                                                        scriptMap.get(i)
                                                                    );

            if(endingBracketMap!=null){

                    stack.pop();

            }

            if(stack.isEmpty()){
                break;
            }


        }

        if(!stack.isEmpty() || !firstStartBracktFound){

            throw new HawkException ("Invalid structure {"+structureName+"}");
        }

        end = i;

        mlc.setStart(start);

        mlc.setEnd(end);

        return mlc;
    }

    public boolean doesMemberExist(String structMember) {

        if(structMember == null || structMember.isEmpty()){

            return false;

        }

        boolean status = false;

        for(Entry<Integer,IScript> entry:this.innerScripts.entrySet()){

            if(entry.getValue() instanceof LocalVarDeclScript){

                LocalVarDeclScript localVarDeclScript = (LocalVarDeclScript)entry.getValue();

                status = localVarDeclScript.getLocalVar().getVariableName().equals(structMember);

            }else if(entry.getValue() instanceof StructureScript){

                StructureScript structureScript = (StructureScript)entry.getValue();

                status = structureScript.getStructVarName().getVariableName().equals(structMember);

            }

        }

        return status;
    }

    @Override
    public boolean isVariable() {
        return false;
    }

}




