


package org.hawk.module.script.type;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.hawk.exception.HawkException;

/**
 *
 * @version 1.0 20 Jul, 2010
 * @author msahu
 */
public class DoubleDataType implements IDataType{

    private static final Pattern INTEGERPATTERN=Pattern.compile("(\\-?\\d*)\\.?0*");
    private double data;

    public DoubleDataType(double data){
        this.data = data;
    }

    public double getData() {
        return data;
    }

    public void setData(double data) {
        this.data = data;
    }

    public boolean isPositiveInteger(){
        boolean status = false;
        Matcher m = INTEGERPATTERN.matcher(this.data+"");
        String tmp = null;
        if(m.matches()){
            tmp = m.group(1);
            int i = Integer.parseInt(tmp);
            status = i>0;
        }else{
            status = false;
        }
        
        return status;
    }

    @Override
    public String toString(){
        Matcher m = INTEGERPATTERN.matcher(this.data+"");
        if(m.matches()){
            return m.group(1);
        }
        return this.data+"";
    }
    

    public IDataType add(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType || dataType instanceof StringDataType)){
            throw new HawkException("invalid datatype");
        }
        if(dataType instanceof StringDataType){
            StringDataType result = new StringDataType(this.toString()+dataType.toString());
            return result;
        }
        DoubleDataType thatDateType = (DoubleDataType)dataType;
        double sum = this.getData()+thatDateType.getData();
        DoubleDataType result = new DoubleDataType(sum);
        result.setData(sum);
        return result;
    }

    public DoubleDataType subtract(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }
        DoubleDataType thatDateType = (DoubleDataType)dataType;
        double diff = this.getData()-thatDateType.getData();
        DoubleDataType result = new DoubleDataType(diff);
        result.setData(diff);
        return result;
    }

    public DoubleDataType multiply(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }
        DoubleDataType thatDateType = (DoubleDataType)dataType;
        double mult = this.getData()*thatDateType.getData();
        DoubleDataType result = new DoubleDataType(mult);
        result.setData(mult);
        return result;
    }

    public DoubleDataType divide(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;
        if(thatDateType.getData() == 0){
            throw new HawkException("Divide by zero exception");
        }
        double div = this.getData()/thatDateType.getData();
        DoubleDataType result = new DoubleDataType(div);
        result.setData(div);
        return result;
    }
    public DoubleDataType modulus(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;
        if(thatDateType.getData() == 0){
            throw new HawkException("Divide by zero exception");
        }
        double div = this.getData()%thatDateType.getData();
        DoubleDataType result = new DoubleDataType(div);
        result.setData(div);
        return result;
    }



    public IDataType copy() throws HawkException {
        IDataType clonedCopy = new DoubleDataType(this.data);
        return clonedCopy;
    }

    public BooleanDataType equalTo(IDataType dataType) throws HawkException {
        if(dataType == null || !( dataType instanceof DoubleDataType || dataType instanceof BooleanDataType)){
            throw new HawkException("invalid datatype");
        }
        if(dataType instanceof BooleanDataType){
            return dataType.equalTo(new BooleanDataType(this.getData()!=0));
        }
        DoubleDataType thatDateType = (DoubleDataType)dataType;
        return new BooleanDataType(this.getData() == thatDateType.getData());
    }

    public BooleanDataType greaterThan(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;
       
        boolean isGreaterThan = this.getData() > thatDateType.getData();
        BooleanDataType result = new BooleanDataType(isGreaterThan);
        result.setData(isGreaterThan);
        return result;
    }

    public BooleanDataType lessThan(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;

        boolean isLessThan = this.getData() < thatDateType.getData();
        BooleanDataType result = new BooleanDataType(isLessThan);
        result.setData(isLessThan);
        return result;
    }

    public BooleanDataType greaterThanEqualTo(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;

        boolean isGreaterThanEqualTo = this.getData() >= thatDateType.getData();
        BooleanDataType result = new BooleanDataType(isGreaterThanEqualTo);
        result.setData(isGreaterThanEqualTo);
        return result;
    }

    public BooleanDataType lessThanEqualTo(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;

        boolean isLessThanEqualTo = this.getData() <= thatDateType.getData();
        BooleanDataType result = new BooleanDataType(isLessThanEqualTo);
        result.setData(isLessThanEqualTo);
        return result;
    }

    public BooleanDataType and(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;

        boolean and = (this.getData()!=0) && (thatDateType.getData()!=0);
        BooleanDataType result = new BooleanDataType(and);
        result.setData(and);
        return result;
    }

    public BooleanDataType or(IDataType dataType) throws HawkException {
        if(dataType == null || !(dataType instanceof DoubleDataType)){
            throw new HawkException("invalid datatype");
        }

        DoubleDataType thatDateType = (DoubleDataType)dataType;

        boolean and = (this.getData()!=0) || (thatDateType.getData()!=0);
        BooleanDataType result = new BooleanDataType(and);
        result.setData(and);
        return result;
    }

    

}




