


package org.hawk.startup;

import java.util.logging.Logger;
import java.io.File;
import org.hawk.module.config.HawkConfigManager;
import org.hawk.module.script.ScriptInterpreter;
import org.hawk.module.script.ScriptUsage;
import org.hawk.module.script.enumeration.BuildModeEnum;
import java.lang.reflect.Method;

import java.util.logging.FileHandler;
import java.util.logging.Level;
import org.hawk.logger.HawkLogger;
import java.util.logging.SimpleFormatter;
import static org.hawk.constant.HawkConstant.*;

/**
 * This is a startup class.
 * <p> Apart from initializing Hawk, it kicks off the startup of<br>
 * target application.<br>
 * Note: The target module should implement <tt>IModule</tt>.
 * @version 1.0 9 Apr, 2010
 * @author msahu
 */
public class HawkStartup {

    private static final Logger rootLogger = Logger.getLogger("");
    private static final HawkLogger logger = HawkLogger.getLogger(HawkStartup.class.getName());
    private static HawkStartup instance = new HawkStartup();


    public static boolean setupLogger(String loggerFile){
        try {
            FileHandler fileHandler = new FileHandler(loggerFile);
            SimpleFormatter simpleFormatter = new SimpleFormatter();
            fileHandler.setFormatter(simpleFormatter);
            rootLogger.addHandler(fileHandler);
        } catch (Exception ex) {
            logger.log(Level.SEVERE,ex.getMessage());
            return false;
        }
        return true;
    }
    
    private HawkStartup(){

    }

    /**
     * SingleTon accessor method
     * @return
     */
    public static HawkStartup getInstance(){
        return instance;
    }
    

    /**
     * This starts the targe app.
     * @return
     */
    public boolean startUp(){
        BuildModeEnum buildMode = ScriptUsage.getInstance().getBuildMode();
        if(buildMode == null || buildMode != BuildModeEnum.PLOTTING){
            HawkConfigManager.cacheConfig();
            ScriptInterpreter.getInstance().cacheSubTasks();
        }

        
        boolean status = true;
        if(buildMode == null || buildMode == BuildModeEnum.PERF){
        
            status  = startupTargetModule();
        }
        if(status){
            String logPath = System.getProperty("LOGPATH");
            if(logPath == null){
                return true;
            }
            File logPathFile = new File(logPath);
            if(!logPathFile.exists()){
                logPathFile.mkdirs();
            }
            String logFile = logPath+"/"+"hawk.log";
            setupLogger(logFile);
        }

        return status;
        
    }

    private boolean startupTargetModule(){

        
        String targetMainModule = HawkConfigManager.getTargetMainModule();
        String targetPackage = HawkConfigManager.getTargetModulePackage();

        String targetModuleClazzStr = targetPackage + SEPARATOR + targetMainModule;
        Class targetModuleClazz = null;
        try{

            targetModuleClazz = Class.forName(targetModuleClazzStr);
            Object targetModuleInstance = targetModuleClazz.newInstance();
            Class noArgs[] = null;
            Method startUpMethod = targetModuleClazz.getMethod("startUp", noArgs);
            startUpMethod.invoke(targetModuleInstance, new Object[]{});
        }catch(Exception ex){
            logger.log(Level.SEVERE,"Failed to invoke startup method of target app", ex);
            System.exit(1);
        }


    
        return true;
        
    }
}




