


package org.hawk.util;

import java.util.Scanner;
import java.io.IOException;
import org.hawk.exception.HawkException;
import org.hawk.ds.Stack;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.util.logging.Level;
import org.hawk.logger.HawkLogger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import static org.hawk.constant.HawkConstant.*;

/**
 *
 * @version 1.0 30 Mar, 2010
 * @author msahu
 */
public class HawkUtil {

   private static final HawkLogger logger = HawkLogger.getLogger(HawkUtil.class.getName());


   public static String readLineFromConsole() throws HawkException{

       String data = null;

       Scanner scanner = new Scanner(System.in);
       data = scanner.nextLine();
       return data;
   }

   public static void sleep(long thinkTime){
       if(thinkTime >0){
       try {
                
                Thread.sleep(thinkTime * 1000);
        } catch (InterruptedException ex) {
            logger.log(Level.SEVERE, ex.getMessage());
        }
       }
   }

   public static boolean getBooleanData(double element) throws HawkException{
       if(!(element == 1.0 || element == 0.0)){
           throw new HawkException("Can not convert element{"+element+"} to boolean.");
       }

       boolean result = false;

       result = (element==1.0)?true:false;

       return result;
   }
   public static boolean isMainThread(){
       String currentThreadName = Thread.currentThread().getName();
       
       return currentThreadName.equals(mainThreadName);
   }
   private static void close(Closeable c){
        if (c != null) {
          try {
            c.close();
          } catch (IOException e) {
            logger.log(Level.SEVERE, e.getMessage());
          }
        }
    }

   public static String convertHawkStringToJavaString(String hawkString){
       if(hawkString == null || hawkString.isEmpty()){
           return hawkString;
       }
       hawkString = hawkString.trim();
       if(hawkString.startsWith("\"") && hawkString.endsWith("\"")){
           return hawkString.substring(1,hawkString.length()-1);
       }else{
           return hawkString;
       }
       

   }

   public static boolean executeProcess(String command){

        if(command == null || command.isEmpty()){
            return false;
        }
        Process proc = null;
        boolean status = true;
        try {
            proc = Runtime.getRuntime().exec(command);
            proc.waitFor();
        }catch(Throwable th){
            logger.log(Level.SEVERE,"Could not execute {"+command+"}",th);
            status = false;
        }finally {
            if (proc != null) {
                close(proc.getOutputStream());
                close(proc.getInputStream());
                close(proc.getErrorStream());
                proc.destroy();
            }
        }
        return status;

   }

   public static boolean isStringDataType(String exp){
        if(exp == null || exp.isEmpty()){
            return false;
        }
        boolean status = false;
        String tmp = exp.trim();
        if(tmp.startsWith("\"") && tmp.endsWith("\"")){
            status = true;
        }
        return status;
    }
    

   public static String readFile(String filePath) throws HawkException {

       StringBuilder sb = new StringBuilder();
       BufferedReader bfr = null;
       try{
            bfr = new BufferedReader(new InputStreamReader(new FileInputStream(filePath)));
            String line = null;
            do{

                line = bfr.readLine();
                if(line!=null){
                    sb.append(line+"\n");
                }
            }while(line!=null);

       }catch(Exception ex){

           logger.log(Level.SEVERE, ex.getMessage());
           return null;
       }finally{
           try{

               bfr.close();

           }catch(Exception ex1){
            logger.log(Level.SEVERE, ex1.getMessage());
            return null;
           }
       }
       return sb.toString();
   }

    public static boolean writeFile(String filePath,String data) throws HawkException {

       
       BufferedWriter bfr = null;
       try{
            bfr = new BufferedWriter(new FileWriter(new File(filePath)));
            bfr.write(data);
            

       }catch(Exception ex){

           logger.log(Level.SEVERE,"error while writing ", ex);
           return false;
       }finally{
           try{

               bfr.close();
               
           }catch(Exception ex1){

               logger.log(Level.SEVERE,"error while closing buffer "+ex1);
               return false;
           }
       }
       return true;
   }

   public static String parseBracketData(String input ) throws HawkException{

        return parseDelimeterData(input, '(', ')');
   }


   public static String parseDelimeterData(String input,char startingDelimChar,char  closingDelimChar) throws HawkException{

        if(input ==null || input.isEmpty()){
            return "";
        }
        String sd = ""+startingDelimChar;
        String cd = ""+closingDelimChar;
        Stack<String> stack = new Stack<String>();
        boolean atLeastOneStartingBracketFound = false;
        int i = 0;
        int start = 0;
        int end = input.length();
        for(;i<input.length();i++){

            String eleStr = input.charAt(i)+"";
            if (!atLeastOneStartingBracketFound && eleStr.equals(sd)){
                stack.push(sd);
                if(!atLeastOneStartingBracketFound){
                    start = i;
                }
                atLeastOneStartingBracketFound = true;

            }else if (eleStr.equals(cd)){
                stack.pop();
            }else if (eleStr.equals(sd)){
                stack.push(sd);
            }
            if(atLeastOneStartingBracketFound && stack.isEmpty()){
                end = i;
                break;
            }
        }
        String data = null;
        if(atLeastOneStartingBracketFound && input.startsWith(sd)){
            data = input.substring(start+1,end);
        }else{
            data= null;
        }
        return data;
   }

   public static void main(String args[]) throws HawkException{


       String x1 = " ` exec SystemModule->Execute(\"ls -ltr\") `";

       String y1 = parseDelimeterData(x1, '`', '`');
       System.out.println(y1);


       String x = "for ( ADF (); ADDDD  ; DFDF ) XXXXXXXX";
       String y = parseBracketData(x);
       System.out.println(y);
       y = y.trim();
       Matcher m = FOR_LOOP_DATA_PATTERN.matcher(y);
       if(m.matches()){
            String varInit = m.group(1);
            String exp = m.group(2);
            String varItr = m.group(3);
            if(!varInit.trim().isEmpty()){
                System.out.println(varInit);
            }else{
                System.out.println("No init");
            }
             if(!exp.trim().isEmpty()){
                System.out.println(exp);
            }else{
                System.out.println("Error");
            }
           
            if(!varItr.trim().isEmpty()){
                System.out.println(varItr);
            }else{
                System.out.println("no check");
            }
       }else{
           System.out.println("screw it");
       }

       

   }

    private static final Pattern FOR_LOOP_DATA_PATTERN=
            Pattern.compile("\\s*(.*)\\s*;\\s*(.*)\\s*;\\s*(.*)\\s*");


    
}




